/***************************************************************************
                          camera.cpp  -  description
                             -------------------
    begin                : Sat Oct 4 2003
    copyright            : (C) 2003 by Marek Rychlik
    email                : marek@rychliks.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "camera.h"

Camera::Camera() : location(0,0,-1),
		   look_at(0,0,0),
		   up(0,1,0),
		   scalex(1.0),
		   scaley(1.0) {
}
Camera::~Camera(){
}
/** Read property of Vec3f location. */
const Vec3f& Camera::getlocation(){
	return location;
}
/** Write property of Vec3f location. */
void Camera::setlocation( const Vec3f& _newVal){
	location = _newVal;
}
/** Read property of Vec3f look_at. */
const Vec3f& Camera::getlook_at(){
	return look_at;
}
/** Write property of Vec3f look_at. */
void Camera::setlook_at( const Vec3f& _newVal){
	look_at = _newVal;
}
/** Read property of Vec3f up. */
const Vec3f& Camera::getup(){
	return up;
}

/** Write property of float angle. */
void Camera::setangle( const float& _newVal){
  setscale(tanf(_newVal));
}

/** Write property of float angle, angle in degrees. */
void Camera::setangle_degrees( const float& _newVal){
  setangle(M_PI*_newVal/360);
}

void Camera::setscale( const float& scale) {
  setscalex(scale);
  setscaley(scale);
}

/** Write property of Vec3f up. */
void Camera::setup( const Vec3f& _newVal){
	up = _newVal;
}
/** No descriptions */
const char* Camera::getname() const{
  return "Camera";
}
/** No descriptions */
ostream & Camera::print(ostream & s) const{
  return s << "location=" << location
	   <<", up=" << up <<", look_at=" << look_at
     <<", scalex=" << scalex
     <<", scaley=" << scaley;
}
/** Construct a ray for a pair of coordinates within [0,1]x[0,1]. */
Ray Camera::getray(float x, float y) const{
  // Fix me!!! This is now a fixed camera with
  // angle = pi/4 and [-1,1]x[-1,1] as viewport
  x = 2 * x - 1;
  y = 2 * y - 1;
  x *= scalex;
  y *= scaley;
  return Ray(location, Vec3f(x, y, 1));  
}
/** Read property of float scalex. */
const float& Camera::getscalex(){
	return scalex;
}
/** Write property of float scalex. */
void Camera::setscalex( const float& _newVal){
	scalex = _newVal;
}

/** Read property of float scaley. */
const float& Camera::getscaley(){
	return scaley;
}
/** Write property of float scaley. */
void Camera::setscaley( const float& _newVal){
	scaley = _newVal;
}
