/***************************************************************************
                          camera.h  -  description
                             -------------------
    begin                : Sat Oct 4 2003
    copyright            : (C) 2003 by Marek Rychlik
    email                : marek@rychliks.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CAMERA_H
#define CAMERA_H


/**
  *@author Marek Rychlik
  */

#include "abstractnode.h"
#include "vec3f.h"
#include "ray.h"
#include <cmath>

class Camera : public AbstractNode{
public: 
	Camera();
	virtual ~Camera();
  /** Write property of Vec3f location. */
  virtual void setlocation( const Vec3f& _newVal);
  /** Read property of Vec3f location. */
 
  virtual const Vec3f& getlocation();
  /** Write property of Vec3f look_at. */
  virtual void setlook_at( const Vec3f& _newVal);
  /** Read property of Vec3f look_at. */
  virtual const Vec3f& getlook_at();
  /** Write property of Vec3f up. */
  virtual void setup( const Vec3f& _newVal);
  /** Read property of Vec3f up. */
  virtual const Vec3f& getup();
  /** Write property of float angle. */
  virtual void setangle( const float& _newVal);
  /** Write property of float angle, angle in degrees. */
  void setangle_degrees( const float& _newVal);
  /** Read property of float angle. */
  virtual const char* getname() const;
  /** No descriptions */
  ostream & print(ostream & s) const;
  /** Construct a ray for a pair of coordinates within [0,1]x[0,1]. */
  Ray getray(float x, float y) const;
  /** Write property of float scalex. */
  virtual void setscalex( const float& _newVal);
  /** Read property of float scalex. */
  virtual const float& getscalex();
  /** No descriptions */
  void setscale(const float& scale);
  /** Write property of float scaley. */
  virtual void setscaley( const float& _newVal);
  /** Read property of float scaley. */
  virtual const float& getscaley();
  
protected: // Public attributes
  /**  */
  Vec3f location;
  /**  */
  Vec3f look_at;
  /**  */
  Vec3f up;
  /**  */
  float scalex;
  /**  */
  float scaley;
};

#endif
