/***************************************************************************
                          plane.cpp  -  description
                             -------------------
    begin                : Thu Oct 2 2003
    copyright            : (C) 2003 by Marek Rychlik
    email                : marek@rychliks.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "plane.h"
#include <cmath>

Plane::Plane() : normal(0,0,1) {
}
Plane::~Plane(){
}
/** Read property of Vec3f normal. */
const Vec3f& Plane::getnormal() const{
	return normal;
}
/** Write property of Vec3f normal. */
void Plane::setnormal( const Vec3f& _newVal){
	normal = _newVal;
  normal.normalize();
}
/** Read property of float distance. */
const float& Plane::getdistance(){
	return distance;
}
/** Write property of float distance. */
void Plane::setdistance( const float& _newVal){
	distance = _newVal;
}


/** No descriptions */
Hit* Plane::reflect(const Ray& ray) const{
  /*
    Solve ray.origin + t * ray.direction  = u
    <u - distance * normal, normal> = 0;
    i.e. <ray.origin - distance * normal> = -t <ray.direction, normal>
  */
  float a = distance - ray.getorigin() * normal;
  float b = ray.getdirection() * normal;
  if(b != 0) {
    float t = a / b;
    if( t > 0) {
      Vec3f hit_location = ray.getorigin() + t * ray.getdirection();
      return new Hit(hit_location, normal);
    } else 
      return 0;
  } else if (a == 0) {
    return 0;
  } else {
    return 0;
  }
}

/** No descriptions */
Plane::Plane(const Vec3f& _normal, const float& _distance) :
  normal(_normal), distance(_distance)
{
   normal.normalize();
}

/** Compute unit normal at a point. */
Vec3f Plane::getnormal(const Vec3f & pt) const{
  return normal;
}

/** No descriptions */
const char* Plane::getname() const{
  return "Plane";
}

ostream& Plane::print(ostream& s) const
{
  return s << "normal=" << normal
	   << ", distance=" << distance;
}

/** No descriptions */
Plane::Plane(const Vec3f& _normal, float _distance) :
  normal(_normal), distance(_distance)
{
  normal.normalize();
}
/** No descriptions */
Plane::Plane(float x, float y, float z, float _distance) :
 normal(Vec3f(x, y, z).normalize()), distance(_distance) {
}
