/***************************************************************************
                          scene.cpp  -  description
                             -------------------
    begin                : Sat Oct 4 2003
    copyright            : (C) 2003 by Marek Rychlik
    email                : marek@rychliks.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "scene.h"
#include <cmath>		// for MAXFLOAT

Scene::Scene(){
}
Scene::~Scene(){
}

LightList& Scene::getlights(){
	return lights;
}
/** Write property of LightList lights. */
void Scene::setlights( const LightList& _newVal){
	lights = _newVal;
}
        
/** Read property of Camera camera. */
Camera& Scene::getcamera(){
	return camera;
}
/** Write property of Camera camera. */
void Scene::setcamera( const Camera& _newVal){
	camera = _newVal;
}
/** No descriptions */
const char * Scene::getname() const{
  return "Scene";
}
/** No descriptions */
ostream & Scene::print(ostream & s) const {
  return s << "camera=" << camera
	   << ", lights=[" << lights
	   << "], shapes=[" << shapes << "]";
}

/** No descriptions */
void Scene::render(Renderer& r) const{
  for(unsigned row = 0; row < r.getheight(); row++) {
    for(unsigned column = 0; column < r.getwidth(); column++) {
      // Construct a ray from the camera corresponding to this pixel
      Ray ray = camera.getray((float) row / r.getheight(), (float) column / r.getwidth());
      Color color = trace(ray);
      r.setpixel(row, column, color);
    }
  }
  
}

#include <stack>
typedef stack<Ray> RayStack;

#include <hit.h>

/** No descriptions */
Color Scene::trace(const Ray& ray) const {
  Color color(Vec3f(0, 0, 0));	// Initialize color to black
  Hit *hit = firstHit(ray);
  if(hit) {
    for(LightList::const_iterator l = lights.begin(); l != lights.end(); l++) {
      // See if the light is visible
      Ray toLight(hit->getlocation(), l->getlocation() - hit->getlocation());
      if(!firstHit(toLight)) {
	// The light is visible
	color += lambert(l->getcolor(), toLight.getdirection(), hit->getnormal());
      }
    }
    delete hit;
    color.calibrate();
  }
  return color;
}

#if !defined(MAXFLOAT)
#define MAXFLOAT ((float) 3.0e37)
#endif

Hit* Scene::firstHit(const Ray& ray) const {
  float distance = MAXFLOAT;
  Hit* retval = 0;
  for(ShapeList::const_iterator shape = shapes.begin(); shape!= shapes.end(); shape++) {
    // Bounce the ray off the shape
    Hit *bounced = shape->reflect(ray);
    // If indeed there was an intersection
    if(bounced) {
      float distance1 = (bounced->getlocation() - ray.getorigin()).length();
      if( (distance > distance1) || !retval) {
	distance = distance1;
	retval = bounced;
      } else {
	delete bounced;
      }
    }
  }
  return retval;
}

Color Scene::lambert(const Color& lightColor, const Vec3f& unitTowardsLight, const Vec3f& normalToSurface) const {
  float factor = unitTowardsLight * normalToSurface;
  if(factor > 0) {
    return factor * lightColor;
  } else {
    return Color(Vec3f(0, 0, 0));
  }
}

/** Read property of ShapeList shapes. */
const ShapeList& Scene::getshapes() const{
	return shapes;
}
/** Write property of ShapeList shapes. */
void Scene::setshapes( const ShapeList& _newVal){
	shapes = _newVal;
}
/** Read property of ShapeList shapes. */
ShapeList & Scene::getshapes(){
  return shapes;
}
/** No descriptions */
void Scene::addshape(AbstractShape *shape){
  shapes.push_front(MaterialShape(shape));
}
/** No descriptions */
void Scene::addpointlight(const Color& color, const Vec3f& location){
  lights.push_front(PointLight(color, location));
}
