/***************************************************************************
                          sphere.cpp  -  description
                             -------------------
    begin                : Thu Oct 2 2003
    copyright            : (C) 2003 by Marek Rychlik
    email                : marek@rychliks.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "sphere.h"
#include <cmath>

Sphere::Sphere(){
}

Sphere::~Sphere(){
}

/** Read property of Vec3f center. */
const Vec3f& Sphere::getcenter(){
	return center;
}

/** Write property of Vec3f center. */
void Sphere::setcenter( const Vec3f& _newVal){
	center = _newVal;
}

/** No descriptions */
 Sphere::Sphere(float x, float y, float z, float r) : center(x, y, z), radius(r){
}

/** Read property of float radius. */
const float& Sphere::getradius(){
	return radius;
}

/** Write property of float radius. */
void Sphere::setradius( const float& _newVal){
	radius = _newVal;
}

/** Get normal at a point */
Vec3f Sphere::getnormal(const Vec3f& pt) const{
  return Vec3f(pt) /= radius;
}

#include <cassert>

/** No descriptions */
Hit* Sphere::reflect(const Ray & ray) const{
   // Solve || ray.origin + t * ray.direction - center ||= radius
   float b = ray.getdirection() * (ray.getorigin() - center);
   float c = (ray.getorigin()-center).lengthsq() - radius * radius;
   float delta =  b * b - c;
   if(delta < 0) {
     // No positive real root
     return 0;
   } else {
     float t = -b - ::sqrtf(delta);
     if(t > 0) {
       Vec3f hit_location = ray.getorigin() + t * ray.getdirection();
       assert(fabsf((hit_location-center).length() - radius) < 1e-4);
       return new Hit(hit_location, getnormal(hit_location - center));
     } else
       return 0;
   }
}

/** No descriptions */
const char * Sphere::getname() const{
  return "Sphere";
}

/** No descriptions */
ostream& Sphere::print(ostream& s) const{
  return s << center << "," << radius;
}
