/***************************************************************************
                          vec3f.cpp  -  description
                             -------------------
    begin                : Thu Oct 2 2003
    copyright            : (C) 2003 by Marek Rychlik
    email                : marek@rychliks.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "vec3f.h"
#include <cmath>
#include <functional>

Vec3f::Vec3f(){
  rep[0]=0;
  rep[1]=0;
  rep[2]=0;
}
Vec3f::~Vec3f(){
}
/** Constructs a new vector. */
Vec3f::Vec3f(float x, float y, float z){
   rep[0] = x;
   rep[1] = y;
   rep[2] = z;
}
/** No descriptions */
const float& Vec3f::getx() const {
  return rep[0];
}
/** No descriptions */
const float& Vec3f::gety() const{
  return rep[1];
}
/** No descriptions */
const float& Vec3f::getz() const{
  return rep[2];
}

Vec3f& Vec3f::operator+=(const Vec3f& x) {
  rep[0] += x.rep[0];
  rep[1] += x.rep[1];
  rep[2] += x.rep[2];
  return *this;
}

Vec3f& Vec3f::operator-=(const Vec3f& x) {
  rep[0] -= x.rep[0];
  rep[1] -= x.rep[1];
  rep[2] -= x.rep[2];
  return *this;
}

Vec3f& Vec3f::operator*=(const float& scalar) {
  rep[0] *= scalar;
  rep[1] *= scalar;
  rep[2] *= scalar;
  return *this;
}

Vec3f& Vec3f::operator/=(const float& scalar) {
  rep[0] /= scalar;
  rep[1] /= scalar;
  rep[2] /= scalar;
  return *this;
}

float Vec3f::operator*(const Vec3f& x) const {
  return rep[0] * x.rep[0] + rep[1] * x.rep[1] + rep[2] * x.rep[2];
}

Vec3f Vec3f::operator+(const Vec3f& x) const {
  Vec3f retval(*this);
  return  retval += x;
}

Vec3f Vec3f::operator-(const Vec3f& x) const {
  Vec3f retval(*this);
  return  retval -= x;
}

Vec3f operator*(const float& scalar, const Vec3f& vector) {
  Vec3f retval(vector);
  return  retval *= scalar;
}
/** No descriptions */
Vec3f& Vec3f::normalize(){
  return *this /= length();
}
/** No descriptions */
float Vec3f::length() const{
  return ::sqrtf(lengthsq());
}

/** No descriptions */
float Vec3f::lengthsq() const{
  return (*this)*(*this);
}

ostream& operator << (ostream& s, const Vec3f& vector) {
  return s << "<" << vector.rep[0] <<","
    << vector.rep[1] << ","
    << vector.rep[2] <<">";
}
/** No descriptions */
Vec3f& Vec3f::cross(const Vec3f& x){
  rep[0] = rep[1]*x.rep[2]-rep[2]*x.rep[1];
  rep[1] = -rep[0]*x.rep[2]+rep[2]*x.rep[0];
  rep[2] = rep[0]*x.rep[1] - rep[1]*x.rep[0];

  return *this;
}
/** No descriptions */
float Vec3f::operator[](int i) const{
  return rep[i];
}

#include <algorithm>		// for min

/** No descriptions */
Vec3f& Vec3f::min(const Vec3f& _v){
  rep[0] = ::min(rep[0], _v.rep[0]);
  rep[1] = ::min(rep[1], _v.rep[1]);
  rep[2] = ::min(rep[2], _v.rep[2]);
  return *this;
}
